<?php
declare(strict_types=1);

namespace Inti\Db\Connection;

use Doctrine\DBAL\Configuration;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DriverManager;
use Doctrine\DBAL\Exception as DbalException;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Class ConnectionAbstractFactory
 *
 * @package Inti\Db\Connection
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-db
 */
final class ConnectionFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param null|array $options
     *
     * @return Connection
     * @noinspection PhpMissingParamTypeInspection
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null): Connection
    {
        $params = $this->getConfig($container, $requestedName);

        if ($params === null) {
            throw new ServiceNotCreatedException(sprintf(
                '%s: Connection `%s` not found in the configuration.',
                __METHOD__, $requestedName
            ));
        }

        $config = new Configuration();

        try {
            return DriverManager::getConnection($params, $config);
        } catch (DbalException $e) {
            throw new ServiceNotCreatedException(sprintf(
                '%s: Could not create connection `%s`.',
                __METHOD__, $requestedName
            ), 0, $e);
        }
    }

    /**
     * @param ContainerInterface $container
     * @param string $name
     * @return array|null
     */
    private function getConfig(ContainerInterface $container, string $name): ?array
    {
        $config = $container->has('config') ? $container->get('config') : [];
        return $config['db_connections'][$name] ?? null;
    }
}
